"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require('./fs');
const cli_engine_config_1 = require("cli-engine-config");
const cli_ux_1 = require("cli-ux");
const heroku_cli_color_1 = require("heroku-cli-color");
const path = require("path");
const deps_1 = require("./deps");
const debug = require('debug')('cli');
const handleEPIPE = (err) => {
    if (err.code !== 'EPIPE')
        throw err;
};
const g = global;
if (!g.testing) {
    process.once('SIGINT', () => {
        if (cli_ux_1.default.action.task)
            cli_ux_1.default.action.stop(heroku_cli_color_1.color.red('ctrl-c'));
        cli_ux_1.default.exit(1);
    });
    let handleErr = (err) => {
        cli_ux_1.default.error(err);
    };
    process.once('uncaughtException', handleErr);
    process.once('unhandledRejection', handleErr);
    process.stdout.on('error', handleEPIPE);
    process.stderr.on('error', handleEPIPE);
}
process.env.CLI_ENGINE_VERSION = require('../package.json').version;
class CLI {
    constructor(config) {
        this.config = config;
    }
    async run(argv) {
        debug('starting run: %o', argv);
        const id = argv[2];
        if (id !== 'update') {
            const updater = new deps_1.default.Updater(this.config);
            await updater.autoupdate();
        }
        const commands = new deps_1.default.CommandManager(this.config);
        const result = await commands.run(argv);
        await this.exitAfterStdoutFlush();
        return result;
    }
    async exitAfterStdoutFlush() {
        if (g.testing)
            return;
        const { timeout } = require('./util');
        await timeout(this.flush(), 10000);
    }
    flush() {
        let p = new Promise(resolve => process.stdout.once('drain', resolve));
        process.stdout.write('');
        return p;
    }
}
exports.default = CLI;
function run(arg1 = process.argv, opts = {}) {
    const argv = Array.isArray(arg1) ? arg1 : opts.argv || process.argv;
    if (!opts.initPath)
        opts.initPath = module.parent.filename;
    if (!opts.root) {
        const findUp = require('find-up');
        opts.root = path.dirname(findUp.sync('package.json', {
            cwd: opts.initPath,
        }));
    }
    const config = cli_engine_config_1.buildConfig(opts);
    if (config.debug)
        cli_ux_1.default.config.debug = true;
    cli_ux_1.default.config.errlog = config.errlog;
    return new CLI(config).run(argv);
}
exports.run = run;
