"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cli_ux_1 = require("cli-ux");
const cli_1 = require("./cli");
async function example(argv) {
    let code = 0;
    argv = ['node', 'cli-engine', ...argv];
    await cli_1.run(argv);
    return {
        stdout: cli_ux_1.cli.stdout.output,
        stderr: cli_ux_1.cli.stderr.output,
        code,
    };
}
exports.example = example;
test('runs the version command', async () => {
    const { stdout } = await example(['version']);
    expect(stdout).toMatch(/^cli-engine\//);
});
test.skip('errors with invalid arguments', async () => {
    expect.assertions(1);
    try {
        await example(['version', '--invalid-flag']);
    }
    catch (err) {
        expect(err.message).toMatch(/^Unexpected argument: --invalid-flag/);
    }
});
test.skip('errors when command not found', async () => {
    expect.assertions(2);
    try {
        await example(['foobar12345']);
    }
    catch (err) {
        expect(cli_ux_1.cli.stderr.output).toMatch(/foobar12345 is not a cli-engine command./);
        expect(err.code).toEqual(127);
    }
});
describe('edge cases', () => {
    test.skip('shows help for `help` command itself', async () => {
        let { stdout } = await example(['help']);
        expect(stdout).toMatch(/Usage: cli-engine COMMAND/);
    });
});
// describe('cli help', () => {
//   describe('global help', () => {
//     let globalHelpOutput = /^Usage: \S+ COMMAND/m
//     test('shows help when no arguments given', async function () {
//       let cli = await run()
//       expect(cli.cmd.out.stdout.output).toMatch(globalHelpOutput)
//     })
//     test('shows help for `help` command and no additonal arguments', async function () {
//       let cli = await run('help')
//       expect(cli.cmd.out.stdout.output).toMatch(globalHelpOutput)
//     })
//     test('shows help for `--help` or `-h` flag and no additonal arguments', async function () {
//       let cli = await run('--help')
//       let clid = await run('-h')
//       expect(cli.cmd.out.stdout.output).toMatch(globalHelpOutput)
//       expect(clid.cmd.out.stdout.output).toMatch(globalHelpOutput)
//     })
//   })
//   describe('--help & -h flags', () => {
//     let pluginsHelpOutput = `Usage: cli-engine plugins [flags]
// list installed plugins
// Flags:
//  --core  show core plugins
// Example:
//     $ cli-engine plugins
//     heroku-ci 1.8.0
//     heroku-cli-status 3.0.10
//     heroku-fork 4.1.22
// cli-engine plugins commands: (get help with cli-engine help plugins:COMMAND)
//  plugins                   list installed plugins
//  plugins:install PLUGIN    installs a plugin into the CLI
//  plugins:link [PATH]       links a local plugin to the CLI for development
//  plugins:uninstall PLUGIN  uninstalls or unlinks a plugin
//  plugins:update            update installed plugins
// `
//     let pluginsInstallHelpOutput = `Usage: cli-engine plugins:install PLUGIN
// installs a plugin into the CLI
// PLUGIN  plugin to install
// Example:
//     $ cli-engine plugins:install heroku-production-status
// `
//     test('shows help for plugins', async function () {
//       let cli = await run('plugins', '--help')
//       let clid = await run('plugins', '-h')
//       expect(cli.cmd.out.stdout.output).toEqual(pluginsHelpOutput)
//       expect(clid.cmd.out.stdout.output).toEqual(pluginsHelpOutput)
//     })
//     test('shows help for plugins (prefixed)', async function () {
//       let cli = await run('--help', 'plugins')
//       let clid = await run('-h', 'plugins')
//       expect(cli.cmd.out.stdout.output).toEqual(pluginsHelpOutput)
//       expect(clid.cmd.out.stdout.output).toEqual(pluginsHelpOutput)
//     })
//     test('shows help for plugins:install', async function () {
//       let cli = await run('plugins:install', 'heroku-sudo', '--help')
//       let clid = await run('plugins:install', 'heroku-sudo', '-h')
//       expect(cli.cmd.out.stdout.output).toEqual(pluginsInstallHelpOutput)
//       expect(clid.cmd.out.stdout.output).toEqual(pluginsInstallHelpOutput)
//     })
//     test('shows help for plugins:install (prefixed)', async function () {
//       let cli = await run('--help', 'plugins:install', 'heroku-sudo')
//       let clid = await run('-h', 'plugins:install', 'heroku-sudo')
//       expect(cli.cmd.out.stdout.output).toEqual(pluginsInstallHelpOutput)
//       expect(clid.cmd.out.stdout.output).toEqual(pluginsInstallHelpOutput)
//     })
//   })
// })
