"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cli_engine_command_1 = require("cli-engine-command");
const cli_ux_1 = require("cli-ux");
const heroku_cli_color_1 = require("heroku-cli-color");
const plugins_1 = require("../../plugins");
const util_1 = require("../../util");
let examplePlugins = {
    'heroku-ci': { version: '1.8.0' },
    'heroku-cli-status': { version: '3.0.10', type: 'link' },
    'heroku-fork': { version: '4.1.22' },
};
let bin = 'heroku';
const g = global;
if (g.config) {
    bin = g.config.bin;
    let pjson = g.config.pjson['cli-engine'];
    if (pjson.help && pjson.help.plugins) {
        examplePlugins = pjson.help.plugins;
    }
}
const examplePluginsHelp = Object.entries(examplePlugins).map(([name, p]) => `    ${name} ${p.version}`);
class default_1 extends cli_engine_command_1.Command {
    async run() {
        let plugins = await this.fetchPlugins();
        plugins = plugins.filter(p => p.type !== 'builtin');
        plugins.sort(util_1.compare('name'));
        if (!this.flags.core)
            plugins = plugins.filter(p => p.type !== 'core');
        if (!plugins.length)
            cli_ux_1.cli.warn('no plugins installed');
        for (let plugin of plugins) {
            let output = `${plugin.name} ${heroku_cli_color_1.color.dim(plugin.version)}`;
            if (plugin.type !== 'user')
                output += heroku_cli_color_1.color.dim(` (${plugin.type})`);
            else if (plugin.tag !== 'latest')
                output += heroku_cli_color_1.color.dim(` (${String(plugin.tag)})`);
            cli_ux_1.cli.log(output);
        }
    }
    fetchPlugins() {
        const plugins = new plugins_1.Plugins(this.config);
        return plugins.list();
    }
}
default_1.topic = 'plugins';
default_1.flags = { core: cli_engine_command_1.flags.boolean({ description: 'show core plugins' }) };
default_1.description = 'list installed plugins';
default_1.help = `Example:
    $ ${bin} plugins
${examplePluginsHelp.join('\n')}
`;
exports.default = default_1;
