"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cli_engine_command_1 = require("cli-engine-command");
const cli_ux_1 = require("cli-ux");
const hooks_1 = require("../../hooks");
const plugins_1 = require("../../plugins");
let examplePlugin = 'heroku-production-status';
let bin = 'heroku';
const g = global;
if (g.config) {
    bin = g.config.bin;
    let pjson = g.config.pjson['cli-engine'];
    if (pjson.help && pjson.help.plugins) {
        examplePlugin = Object.keys(pjson.help.plugins)[0];
    }
}
class PluginsInstall extends cli_engine_command_1.Command {
    async run() {
        const plugins = new plugins_1.Plugins(this.config);
        const [plugin, tag = 'latest'] = this.argv[0].split('@');
        if (!this.config.debug)
            cli_ux_1.cli.action.start(`Installing plugin ${plugin}${tag === 'latest' ? '' : '@' + tag}`);
        await plugins.install({ type: 'user', name: plugin, tag, force: this.flags.force });
        const hooks = new hooks_1.Hooks(this.config);
        await hooks.run('update');
    }
}
PluginsInstall.topic = 'plugins';
PluginsInstall.command = 'install';
PluginsInstall.description = 'installs a plugin into the CLI';
PluginsInstall.help = `
  Example:
    $ ${bin} plugins:install ${examplePlugin}
  `;
PluginsInstall.args = [{ name: 'plugin', description: 'plugin to install' }];
PluginsInstall.flags = {
    force: cli_engine_command_1.flags.boolean({ char: 'f' }),
};
exports.default = PluginsInstall;
