"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cli_engine_command_1 = require("cli-engine-command");
const cli_ux_1 = require("cli-ux");
const path = require("path");
const hooks_1 = require("../../hooks");
const plugins_1 = require("../../plugins");
let bin = 'heroku';
const g = global;
if (g.config) {
    bin = g.config.bin;
}
class PluginsLink extends cli_engine_command_1.Command {
    async run() {
        const plugins = new plugins_1.Plugins(this.config);
        let p = path.resolve(this.argv[0] || process.cwd());
        cli_ux_1.cli.action.start(`Linking ${p}`);
        await plugins.install({ type: 'link', root: p, force: this.flags.force });
        const hooks = new hooks_1.Hooks(this.config);
        await hooks.run('update');
    }
}
PluginsLink.topic = 'plugins';
PluginsLink.command = 'link';
PluginsLink.flags = {
    force: cli_engine_command_1.flags.boolean({ char: 'f' }),
};
PluginsLink.args = [{ name: 'path', optional: true, description: 'local file path to plugin root' }];
PluginsLink.description = 'links a local plugin to the CLI for development';
PluginsLink.help = `Example:
    $ ${bin} plugins:link .
    Installing dependencies for /Users/dickeyxxx/src/github.com/heroku/heroku-status... done
    Running prepare script for /Users/dickeyxxx/src/github.com/heroku/heroku-status... done`;
exports.default = PluginsLink;
