"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cli_engine_command_1 = require("cli-engine-command");
const cli_ux_1 = require("cli-ux");
const heroku_cli_color_1 = require("heroku-cli-color");
const path = require("path");
const hooks_1 = require("../hooks");
const updater_1 = require("../updater");
const update_1 = require("./plugins/update");
const debug = require('debug')('cli:update');
function brew(...args) {
    const cp = require('child_process');
    debug('brew %o', args);
    return cp.spawnSync('brew', args, { stdio: 'inherit' });
}
const g = global;
const cliBin = g.config ? g.config.bin : 'heroku';
class Update extends cli_engine_command_1.Command {
    async run() {
        // on manual run, also log to file
        if (!this.flags.autoupdate) {
            cli_ux_1.default.config.errlog = path.join(this.config.cacheDir, 'autoupdate');
        }
        this.updater = new updater_1.Updater(this.config);
        if (this.config.updateDisabled === 'Update CLI with `brew upgrade heroku`') {
            this.migrateBrew();
        }
        else if (this.config.updateDisabled) {
            cli_ux_1.default.warn(this.config.updateDisabled);
        }
        else {
            cli_ux_1.default.action.start(`${this.config.name}: Updating CLI`);
            let channel = this.argv[0] || this.config.channel;
            let manifest = await this.updater.fetchManifest(channel);
            if (this.config.version === manifest.version && channel === this.config.channel) {
                if (!process.env.CLI_ENGINE_HIDE_UPDATED_MESSAGE) {
                    cli_ux_1.default.action.stop(`already on latest version: ${this.config.version}`);
                }
            }
            else {
                cli_ux_1.default.action.start(`${this.config.name}: Updating CLI from ${heroku_cli_color_1.color.green(this.config.version)} to ${heroku_cli_color_1.color.green(manifest.version)}${channel === 'stable' ? '' : ' (' + heroku_cli_color_1.color.yellow(channel) + ')'}`);
                await this.updater.update(manifest);
                cli_ux_1.default.action.stop();
                try {
                    await this.updater.autoupdate(true);
                    cli_ux_1.default.exit(0);
                }
                catch (err) {
                    cli_ux_1.default.warn(err, { context: 'post-install autoupdate failed' });
                }
            }
        }
        debug('fetch version');
        await this.updater.fetchVersion(true);
        debug('plugins update');
        await update_1.default.run([], this.config);
        debug('log chop');
        await this.logChop();
        debug('tidy');
        await this.updater.tidy();
        const hooks = new hooks_1.Hooks(this.config);
        await hooks.run('update');
        debug('done');
        cli_ux_1.default.action.stop();
    }
    async logChop() {
        try {
            const logChopper = require('log-chopper').default;
            await logChopper.chop(this.config.errlog);
        }
        catch (e) {
            debug(e.message);
        }
    }
    migrateBrew() {
        try {
            debug('migrating from brew');
            const fs = require('fs-extra');
            let p = fs.realpathSync('/usr/local/bin/heroku');
            if (p.match(/\/usr\/local\/Cellar\/heroku\/\d+\.\d+\.\d+\//)) {
                // not on private tap, move to it
                cli_ux_1.default.action.start('Upgrading homebrew formula');
                brew('tap', 'heroku/brew');
                brew('upgrade', 'heroku/brew/heroku');
                cli_ux_1.default.action.stop();
            }
        }
        catch (err) {
            debug(err);
        }
    }
}
Update.topic = 'update';
Update.description = `update the ${cliBin} CLI`;
Update.args = [{ name: 'channel', optional: true }];
Update.flags = {
    autoupdate: cli_engine_command_1.flags.boolean({ hidden: true }),
};
exports.default = Update;
