"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const path = require("path");
const deps_1 = require("./deps");
const debug = require('debug')('cli:file');
function exists(f) {
    // debug('exists', f)
    // @ts-ignore
    return fs.exists(f);
}
exports.exists = exists;
const jsonFiles = {};
function fetchJSONFile(f) {
    if (!jsonFiles[f]) {
        debug('fetchJSONFile', f);
        jsonFiles[f] = fs.readJSON(f);
    }
    return jsonFiles[f];
}
exports.fetchJSONFile = fetchJSONFile;
async function rename(from, to) {
    debug('rename', from, to);
    return fs.rename(from, to);
}
exports.rename = rename;
async function mkdirp(dir) {
    debug('mkdirp', dir);
    return fs.mkdirp(dir);
}
exports.mkdirp = mkdirp;
async function outputFile(file, data, options) {
    debug('outputFile', file);
    return fs.outputFile(file, data, options);
}
exports.outputFile = outputFile;
async function outputJSON(file, data, options = {}) {
    debug('outputJSON', file);
    return fs.outputJSON(file, data, Object.assign({ spaces: 2 }, options));
}
exports.outputJSON = outputJSON;
async function readJSON(file) {
    debug('readJSON', file);
    return fs.readJSON(file);
}
exports.readJSON = readJSON;
async function read(file) {
    debug('read', file);
    return fs.readFile(file, 'utf8');
}
exports.read = read;
async function remove(file) {
    debug('remove', file);
    return fs.remove(file);
}
exports.remove = remove;
async function stat(file) {
    return fs.stat(file);
}
exports.stat = stat;
function open(path, flags, mode) {
    debug('open', path, flags, mode);
    return fs.open(path, flags, mode);
}
exports.open = open;
function write(fd, data) {
    debug('write', fd);
    // @ts-ignore
    return fs.write(fd, data);
}
exports.write = write;
function walk(root, opts = {}) {
    debug('walk', root);
    return new Promise((resolve, reject) => {
        const items = [];
        deps_1.default
            .klaw(root, Object.assign({}, opts, { depthLimit: 10000 }))
            .on('data', f => items.push(f))
            .on('error', reject)
            .on('end', () => resolve(items));
    });
}
exports.walk = walk;
async function ls(dir) {
    let files = await fs.readdir(dir);
    let paths = files.map(f => path.join(dir, f));
    return Promise.all(paths.map(path => fs.stat(path).then(stat => ({ path, stat }))));
}
exports.ls = ls;
async function cleanup(dir) {
    let files;
    try {
        files = await ls(dir);
    }
    catch (err) {
        if (err.code === 'ENOENT')
            return;
        throw err;
    }
    let dirs = files.filter(f => f.stat.isDirectory()).map(f => f.path);
    for (let p of dirs.map(cleanup))
        await p;
    files = await ls(dir);
    if (!files.length)
        await remove(dir);
}
exports.cleanup = cleanup;
async function newestFileInDir(dir) {
    let files = await walk(dir);
    return files.reduce((prev, f) => {
        if (f.stats.isDirectory())
            return prev;
        if (f.stats.mtime > prev)
            return f.stats.mtime;
        return prev;
    }, new Date(0));
}
exports.newestFileInDir = newestFileInDir;
function symlink(src, dst) {
    return fs.symlink(src, dst);
}
exports.symlink = symlink;
