"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cli_ux_1 = require("cli-ux");
const path = require("path");
const deps_1 = require("./deps");
class Lock {
    constructor(config, lockfile) {
        this.config = config;
        this.lockfile = lockfile || path.join(this.config.cacheDir, 'update.lock');
    }
    // get read lock
    async read() {
        return deps_1.default.rwlockfile.read(this.lockfile);
    }
    async unread() {
        await deps_1.default.rwlockfile.unread(this.lockfile);
    }
    async canRead() {
        let hasWriter = await deps_1.default.rwlockfile.hasWriter(this.lockfile);
        return !hasWriter;
    }
    async write() {
        return this.upgrade();
    }
    // upgrade to writer
    async upgrade() {
        // take off reader
        await this.unread();
        // check for other readers
        if (await deps_1.default.rwlockfile.hasReaders(this.lockfile)) {
            cli_ux_1.cli.action.status = `Waiting for all commands to finish`;
        }
        // grab writer lock
        let unlock = await deps_1.default.rwlockfile.write(this.lockfile);
        // return downgrade function
        return async () => {
            // turn back into reader when unlocking
            await unlock();
            return deps_1.default.rwlockfile.read(this.lockfile);
        };
    }
}
exports.Lock = Lock;
