"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cli_engine_command_1 = require("cli-engine-command");
const cli_ux_1 = require("cli-ux");
const heroku_cli_color_1 = require("heroku-cli-color");
const util_1 = require("util");
const deps_1 = require("../deps");
const debug = require('debug')('cli:legacy');
class PluginLegacy {
    constructor(_) { }
    convert(m) {
        m.commands = this.convertCommands(m.commands);
        return m;
    }
    convertCommands(c) {
        return c.map(c => this.convertCommand(c));
    }
    convertCommand(c) {
        if (this.isICommand(c))
            return c;
        if (this.isV5Command(c))
            return this.convertFromV5(c);
        if (this.isFlowCommand(c))
            return this.convertFromFlow(c);
        debug(c);
        throw new Error(`Invalid command: ${util_1.inspect(c)}`);
    }
    convertFromFlow(c) {
        c._version = '0.0.0';
        return c;
    }
    convertFromV5(c) {
        class V5 extends deps_1.default.Heroku.Command {
            async run() {
                if (c.aliases && c.aliases.length) {
                    cli_ux_1.cli.warn(`Using aliases: ${c.aliases}`);
                }
                const ctx = {
                    version: this.config.userAgent,
                    supportsColor: heroku_cli_color_1.color.enabled,
                    auth: {},
                    debug: !!this.config.debug,
                    debugHeaders: this.config.debug > 1 || ['1', 'true'].includes(process.env.HEROKU_DEBUG_HEADERS),
                    flags: this.flags,
                    args: c.variableArgs ? this.argv : this.args,
                    app: this.flags.app,
                    org: this.flags.org,
                    team: this.flags.team,
                    config: this.config,
                    apiUrl: deps_1.default.Heroku.vars.apiUrl,
                    herokuDir: this.config.cacheDir,
                    apiToken: this.heroku.auth,
                    apiHost: deps_1.default.Heroku.vars.apiHost,
                    gitHost: deps_1.default.Heroku.vars.gitHost,
                    httpGitHost: deps_1.default.Heroku.vars.httpGitHost,
                    cwd: process.cwd(),
                };
                ctx.auth.password = ctx.apiToken;
                const ansi = require('ansi-escapes');
                process.once('exit', () => {
                    if (process.stderr.isTTY) {
                        process.stderr.write(ansi.cursorShow);
                    }
                });
                return c.run(ctx);
            }
        }
        V5.topic = c.topic;
        V5.command = c.command;
        V5.description = c.description;
        V5.hidden = !!c.hidden;
        V5.args = (c.args || []).map(a => (Object.assign({}, a, { required: a.required !== false && !a.optional })));
        V5.flags = convertFlagsFromV5(c.flags);
        V5.variableArgs = !!c.variableArgs;
        V5.help = c.help;
        V5.usage = c.usage;
        if (c.needsApp || c.wantsApp) {
            V5.flags.app = deps_1.default.Heroku.flags.app({ required: !!c.needsApp });
            V5.flags.remote = deps_1.default.Heroku.flags.remote();
        }
        if (c.needsOrg || c.wantsOrg) {
            let opts = { required: !!c.needsOrg, hidden: false, description: 'organization to use' };
            V5.flags.org = deps_1.default.Heroku.flags.org(opts);
        }
        return V5;
    }
    isICommand(command) {
        let c = command;
        return !!(c.id && c._version);
    }
    isV5Command(command) {
        let c = command;
        return !!(typeof c === 'object');
    }
    isFlowCommand(command) {
        let c = command;
        return !!(!('_version' in c) && c.id);
    }
}
exports.PluginLegacy = PluginLegacy;
function convertFlagsFromV5(flags) {
    if (!flags)
        return {};
    if (!Array.isArray(flags))
        return flags;
    return flags.reduce((flags, flag) => {
        let opts = {
            char: flag.char,
            description: flag.description,
            hidden: flag.hidden,
            required: flag.required || flag.optional === false,
            parse: flag.parse,
        };
        for (let [k, v] of Object.entries(opts)) {
            if (v === undefined)
                delete opts[k];
        }
        if (!opts.parse)
            delete opts.parse;
        flags[flag.name] = flag.hasValue ? cli_engine_command_1.flags.string(opts) : cli_engine_command_1.flags.boolean(opts);
        return flags;
    }, {});
}
