import { ICommand, IConfig } from 'cli-engine-config';
import { ICommandInfo, ICommandManager, ILoadResult } from '../command';
import { Lock } from '../lock';
export declare type PluginType = 'builtin' | 'core' | 'user' | 'link';
export interface IPluginPJSON {
    name: string;
    version: string;
    main?: string;
    scripts?: {
        [k: string]: string;
    };
    'cli-engine': {
        commandsDir?: string;
        aliases?: {
            [k: string]: string | string[];
        };
    };
}
export interface IPluginTopic {
    name: string;
    description?: string;
    hidden?: boolean;
}
export interface IPluginModule {
    commands: ICommand[];
    topic?: IPluginTopic;
    topics: IPluginTopic[];
}
export interface IPluginOptions {
    config: IConfig;
    root: string;
    pjson?: IPluginPJSON;
}
export declare abstract class Plugin implements ICommandManager {
    abstract type: PluginType;
    name: string;
    version: string;
    tag?: string;
    pjson: IPluginPJSON;
    root: string;
    protected config: IConfig;
    protected debug: any;
    protected lock: Lock;
    private _module;
    private cache;
    constructor(opts: IPluginOptions);
    load(): Promise<ILoadResult>;
    yarnNodeVersion(): Promise<string | undefined>;
    resetCache(): Promise<void>;
    init(): Promise<void>;
    findCommand(id: string, must: true): Promise<ICommand>;
    findCommand(id: string, must?: boolean): Promise<ICommand | undefined>;
    protected commands(): Promise<ICommandInfo[]>;
    protected readonly commandsDir: string | undefined;
    private commandPath(id);
    private commandsFromModule();
    private commandsFromDir();
    private commandInfoFromICommand(icommand);
    private findCommandInDir(id);
    private findCommandInModule(id);
    private addPluginToCommand(cmd);
    private commandIDsFromDir();
    private fetchModule();
}
