"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const topic_1 = require("./topic");
test('it adds topics', () => {
    let root = new topic_1.RootTopic();
    root.addTopics([
        { description: 'bar:baz:bak:fo desc', name: 'bar:baz:bak:fo' },
        { description: 'basic', name: 'basic' },
        { description: 'foo desc', name: 'foo' },
        { description: 'foo:bar desc', name: 'foo:bar' },
        { description: 'foo:bar:baz:bak desc', name: 'foo:bar:baz:bak' },
        { description: 'foo:fee:fie:fo desc', name: 'foo:fee:fie:fo' },
    ]);
    expect(root.findTopic('basic')).toMatchObject({
        description: 'basic',
        name: 'basic',
    });
    expect(root.findTopic('foo:bar:baz:bak')).toMatchObject({
        description: 'foo:bar:baz:bak desc',
        name: 'foo:bar:baz:bak',
    });
});
test('it adds commands', () => {
    const root = new topic_1.RootTopic();
    const basecmd = {
        help: 'str',
        description: 'lkj',
        usage: 'lkj',
        helpLine: ['foo', 'bar'],
        hidden: false,
        id: 'foo:bar:baz:bak',
        run: async () => { },
        aliases: [],
        plugin: {},
    };
    root.addCommands([
        Object.assign({}, basecmd, { id: 'foo:bar:baz:bak', help: 'foo help' }),
        Object.assign({}, basecmd, { id: 'status', help: 'status help' }),
    ]);
    expect(root.findTopic('foo:bar:baz').commands.bak).toHaveProperty('help', 'foo help');
    expect(root.findCommand('foo:bar:baz:bak')).toHaveProperty('help', 'foo help');
    expect(root.findCommand('status')).toHaveProperty('help', 'status help');
});
test('topic merging', () => {
    const root = new topic_1.RootTopic();
    root.addTopics([
        { name: 'foo:bar:buz', description: 'overwritten desc' },
        { name: 'foo:bar', subtopics: { buz: { description: 'buz desc' } } },
    ]);
    expect(root.findTopic('foo:bar:buz')).toHaveProperty('description', 'buz desc');
});
