"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const deps_1 = require("./deps");
const debug = require('debug')('util');
function compare(...props) {
    return (a, b) => {
        for (let prop of props) {
            if (a[prop] === undefined)
                return -1;
            if (b[prop] === undefined)
                return 1;
            if (a[prop] < b[prop])
                return -1;
            if (a[prop] > b[prop])
                return 1;
        }
        return 0;
    };
}
exports.compare = compare;
function wait(ms, unref = false) {
    return new Promise(resolve => {
        let t = setTimeout(resolve, ms);
        if (unref)
            t.unref();
    });
}
exports.wait = wait;
function timeout(p, ms) {
    return Promise.race([p, wait(ms, true).then(() => debug('timed out'))]);
}
exports.timeout = timeout;
function undefault(obj) {
    if (obj.__esModule === true)
        return obj.default;
    return obj;
}
exports.undefault = undefault;
function isEmpty(obj) {
    // null and undefined are "empty"
    if (obj == null)
        return true;
    // Assume if it has a length property with a non-zero value
    // that that property is correct.
    if (obj.length && obj.length > 0)
        return false;
    if (obj.length === 0)
        return true;
    // Otherwise, does it have any properties of its own?
    // Note that this doesn't handle
    // toString and toValue enumeration bugs in IE < 9
    for (let key in obj) {
        if (obj.hasOwnProperty(key))
            return false;
    }
    return true;
}
exports.isEmpty = isEmpty;
function toArray(o) {
    return Array.isArray(o) ? o : [o];
}
exports.toArray = toArray;
function objValsToArrays(input) {
    return Object.entries(input || {}).reduce((output, [k, v]) => {
        output[k] = toArray(v);
        return output;
    }, {});
}
exports.objValsToArrays = objValsToArrays;
function minorVersionGreater(fromString, toString) {
    const from = deps_1.default.semver.parse(fromString);
    const to = deps_1.default.semver.parse(toString);
    if (from.major < to.major)
        return true;
    if (from.major === to.major && from.minor < to.minor)
        return true;
    return false;
}
exports.minorVersionGreater = minorVersionGreater;
